// This file is autogenerated. Changes will be overwritten.
#include "GLIBJZZR4B/moc_qdbusmenuconnection_p.cpp"
#include "GLIBJZZR4B/moc_qdbustrayicon_p.cpp"
#include "GLIBJZZR4B/moc_qstatusnotifieritemadaptor_p.cpp"
#include "EWIEGA46WW/moc_dthemesettings.cpp"
