/****************************************************************************
** Meta object code from reading C++ file 'qstatusnotifieritemadaptor_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../3rdparty/qstatusnotifieritemadaptor_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qstatusnotifieritemadaptor_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_thirdparty__QStatusNotifierItemAdaptor_t {
    QByteArrayData data[39];
    char stringdata0[2973];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_thirdparty__QStatusNotifierItemAdaptor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_thirdparty__QStatusNotifierItemAdaptor_t qt_meta_stringdata_thirdparty__QStatusNotifierItemAdaptor = {
    {
QT_MOC_LITERAL(0, 0, 38), // "thirdparty::QStatusNotifierIt..."
QT_MOC_LITERAL(1, 39, 15), // "D-Bus Interface"
QT_MOC_LITERAL(2, 55, 26), // "org.kde.StatusNotifierItem"
QT_MOC_LITERAL(3, 82, 19), // "D-Bus Introspection"
QT_MOC_LITERAL(4, 102, 2498), // "  <interface name=\"org.kde.S..."
QT_MOC_LITERAL(5, 2362, 16), // "NewAttentionIcon"
QT_MOC_LITERAL(6, 2379, 0), // ""
QT_MOC_LITERAL(7, 2380, 7), // "NewIcon"
QT_MOC_LITERAL(8, 2388, 14), // "NewOverlayIcon"
QT_MOC_LITERAL(9, 2403, 7), // "NewMenu"
QT_MOC_LITERAL(10, 2411, 9), // "NewStatus"
QT_MOC_LITERAL(11, 2421, 6), // "status"
QT_MOC_LITERAL(12, 2428, 8), // "NewTitle"
QT_MOC_LITERAL(13, 2437, 10), // "NewToolTip"
QT_MOC_LITERAL(14, 2448, 8), // "Activate"
QT_MOC_LITERAL(15, 2457, 1), // "x"
QT_MOC_LITERAL(16, 2459, 1), // "y"
QT_MOC_LITERAL(17, 2461, 11), // "ContextMenu"
QT_MOC_LITERAL(18, 2473, 6), // "Scroll"
QT_MOC_LITERAL(19, 2480, 5), // "delta"
QT_MOC_LITERAL(20, 2486, 11), // "orientation"
QT_MOC_LITERAL(21, 2498, 17), // "SecondaryActivate"
QT_MOC_LITERAL(22, 2516, 17), // "AttentionIconName"
QT_MOC_LITERAL(23, 2534, 19), // "AttentionIconPixmap"
QT_MOC_LITERAL(24, 2554, 19), // "QXdgDBusImageVector"
QT_MOC_LITERAL(25, 2574, 18), // "AttentionMovieName"
QT_MOC_LITERAL(26, 2593, 8), // "Category"
QT_MOC_LITERAL(27, 2602, 8), // "IconName"
QT_MOC_LITERAL(28, 2611, 10), // "IconPixmap"
QT_MOC_LITERAL(29, 2622, 2), // "Id"
QT_MOC_LITERAL(30, 2625, 10), // "ItemIsMenu"
QT_MOC_LITERAL(31, 2636, 4), // "Menu"
QT_MOC_LITERAL(32, 2641, 15), // "QDBusObjectPath"
QT_MOC_LITERAL(33, 2657, 15), // "OverlayIconName"
QT_MOC_LITERAL(34, 2673, 17), // "OverlayIconPixmap"
QT_MOC_LITERAL(35, 2691, 6), // "Status"
QT_MOC_LITERAL(36, 2698, 5), // "Title"
QT_MOC_LITERAL(37, 2704, 7), // "ToolTip"
QT_MOC_LITERAL(38, 2712, 21) // "QXdgDBusToolTipStruct"

    },
    "thirdparty::QStatusNotifierItemAdaptor\0"
    "D-Bus Interface\0org.kde.StatusNotifierItem\0"
    "D-Bus Introspection\0"
    "  <interface name=\"org.kde.StatusNotifierItem\">\n    <property acces"
    "s=\"read\" type=\"s\" name=\"Category\"/>\n    <property access=\"read"
    "\" type=\"s\" name=\"Id\"/>\n    <property access=\"read\" type=\"s\" "
    "name=\"Title\"/>\n    <property access=\"read\" type=\"s\" name=\"Stat"
    "us\"/>\n    <property access=\"read\" type=\"i\" name=\"WindowId\"/>\n"
    "    <property access=\"read\" type=\"s\" name=\"IconThemePath\"/>\n   "
    " <property access=\"read\" type=\"o\" name=\"Menu\"/>\n    <property a"
    "ccess=\"read\" type=\"b\" name=\"ItemIsMenu\"/>\n    <property access="
    "\"read\" type=\"s\" name=\"IconName\"/>\n    <property access=\"read\""
    " type=\"a(iiay)\" name=\"IconPixmap\">\n      <annotation value=\"QXdg"
    "DBusImageVector\" name=\"org.qtproject.QtDBus.QtTypeName\"/>\n    </pr"
    "operty>\n    <property access=\"read\" type=\"s\" name=\"OverlayIconNa"
    "me\"/>\n    <property access=\"read\" type=\"a(iiay)\" name=\"OverlayI"
    "conPixmap\">\n      <annotation value=\"QXdgDBusImageVector\" name=\"o"
    "rg.qtproject.QtDBus.QtTypeName\"/>\n    </property>\n    <property acc"
    "ess=\"read\" type=\"s\" name=\"AttentionIconName\"/>\n    <property ac"
    "cess=\"read\" type=\"a(iiay)\" name=\"AttentionIconPixmap\">\n      <a"
    "nnotation value=\"QXdgDBusImageVector\" name=\"org.qtproject.QtDBus.Qt"
    "TypeName\"/>\n    </property>\n    <property access=\"read\" type=\"s\""
    " name=\"AttentionMovieName\"/>\n    <property access=\"read\" type=\"("
    "sa(iiay)ss)\" name=\"ToolTip\">\n      <annotation value=\"QXdgDBusToo"
    "lTipStruct\" name=\"org.qtproject.QtDBus.QtTypeName\"/>\n    </propert"
    "y>\n    <method name=\"ContextMenu\">\n      <arg direction=\"in\" typ"
    "e=\"i\" name=\"x\"/>\n      <arg direction=\"in\" type=\"i\" name=\"y\""
    "/>\n    </method>\n    <method name=\"Activate\">\n      <arg directio"
    "n=\"in\" type=\"i\" name=\"x\"/>\n      <arg direction=\"in\" type=\"i"
    "\" name=\"y\"/>\n    </method>\n    <method name=\"SecondaryActivate\""
    ">\n      <arg direction=\"in\" type=\"i\" name=\"x\"/>\n      <arg dir"
    "ection=\"in\" type=\"i\" name=\"y\"/>\n    </method>\n    <method name"
    "=\"Scroll\">\n      <arg direction=\"in\" type=\"i\" name=\"delta\"/>\n"
    "      <arg direction=\"in\" type=\"s\" name=\"orientation\"/>\n    </m"
    "ethod>\n    <signal name=\"NewTitle\"/>\n    <signal name=\"NewIcon\"/"
    ">\n    <signal name=\"NewAttentionIcon\"/>\n    <signal name=\"NewOver"
    "layIcon\"/>\n    <signal name=\"NewMenu\"/>\n    <signal name=\"NewToo"
    "lTip\"/>\n    <signal name=\"NewStatus\">\n      <arg type=\"s\" name="
    "\"status\"/>\n    </signal>\n  </interface>\n\0"
    "NewAttentionIcon\0\0NewIcon\0NewOverlayIcon\0"
    "NewMenu\0NewStatus\0status\0NewTitle\0"
    "NewToolTip\0Activate\0x\0y\0ContextMenu\0"
    "Scroll\0delta\0orientation\0SecondaryActivate\0"
    "AttentionIconName\0AttentionIconPixmap\0"
    "QXdgDBusImageVector\0AttentionMovieName\0"
    "Category\0IconName\0IconPixmap\0Id\0"
    "ItemIsMenu\0Menu\0QDBusObjectPath\0"
    "OverlayIconName\0OverlayIconPixmap\0"
    "Status\0Title\0ToolTip\0QXdgDBusToolTipStruct"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_thirdparty__QStatusNotifierItemAdaptor[] = {

 // content:
       8,       // revision
       0,       // classname
       2,   14, // classinfo
      11,   18, // methods
      14,  102, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // classinfo: key, value
       1,    2,
       3,    4,

 // signals: name, argc, parameters, tag, flags
       5,    0,   73,    6, 0x06 /* Public */,
       7,    0,   74,    6, 0x06 /* Public */,
       8,    0,   75,    6, 0x06 /* Public */,
       9,    0,   76,    6, 0x06 /* Public */,
      10,    1,   77,    6, 0x06 /* Public */,
      12,    0,   80,    6, 0x06 /* Public */,
      13,    0,   81,    6, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      14,    2,   82,    6, 0x0a /* Public */,
      17,    2,   87,    6, 0x0a /* Public */,
      18,    2,   92,    6, 0x0a /* Public */,
      21,    2,   97,    6, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   11,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   15,   16,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   15,   16,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,   19,   20,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   15,   16,

 // properties: name, type, flags
      22, QMetaType::QString, 0x00095001,
      23, 0x80000000 | 24, 0x00095009,
      25, QMetaType::QString, 0x00095001,
      26, QMetaType::QString, 0x00095001,
      27, QMetaType::QString, 0x00095001,
      28, 0x80000000 | 24, 0x00095009,
      29, QMetaType::QString, 0x00095001,
      30, QMetaType::Bool, 0x00095001,
      31, 0x80000000 | 32, 0x00095009,
      33, QMetaType::QString, 0x00095001,
      34, 0x80000000 | 24, 0x00095009,
      35, QMetaType::QString, 0x00095001,
      36, QMetaType::QString, 0x00095001,
      37, 0x80000000 | 38, 0x00095009,

       0        // eod
};

void thirdparty::QStatusNotifierItemAdaptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QStatusNotifierItemAdaptor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->NewAttentionIcon(); break;
        case 1: _t->NewIcon(); break;
        case 2: _t->NewOverlayIcon(); break;
        case 3: _t->NewMenu(); break;
        case 4: _t->NewStatus((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->NewTitle(); break;
        case 6: _t->NewToolTip(); break;
        case 7: _t->Activate((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 8: _t->ContextMenu((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 9: _t->Scroll((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 10: _t->SecondaryActivate((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QStatusNotifierItemAdaptor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QStatusNotifierItemAdaptor::NewAttentionIcon)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QStatusNotifierItemAdaptor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QStatusNotifierItemAdaptor::NewIcon)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QStatusNotifierItemAdaptor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QStatusNotifierItemAdaptor::NewOverlayIcon)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QStatusNotifierItemAdaptor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QStatusNotifierItemAdaptor::NewMenu)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QStatusNotifierItemAdaptor::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QStatusNotifierItemAdaptor::NewStatus)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QStatusNotifierItemAdaptor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QStatusNotifierItemAdaptor::NewTitle)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QStatusNotifierItemAdaptor::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QStatusNotifierItemAdaptor::NewToolTip)) {
                *result = 6;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 8:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
        case 10:
        case 5:
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QXdgDBusImageVector >(); break;
        case 13:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QXdgDBusToolTipStruct >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QStatusNotifierItemAdaptor *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->attentionIconName(); break;
        case 1: *reinterpret_cast< QXdgDBusImageVector*>(_v) = _t->attentionIconPixmap(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->attentionMovieName(); break;
        case 3: *reinterpret_cast< QString*>(_v) = _t->category(); break;
        case 4: *reinterpret_cast< QString*>(_v) = _t->iconName(); break;
        case 5: *reinterpret_cast< QXdgDBusImageVector*>(_v) = _t->iconPixmap(); break;
        case 6: *reinterpret_cast< QString*>(_v) = _t->id(); break;
        case 7: *reinterpret_cast< bool*>(_v) = _t->itemIsMenu(); break;
        case 8: *reinterpret_cast< QDBusObjectPath*>(_v) = _t->menu(); break;
        case 9: *reinterpret_cast< QString*>(_v) = _t->overlayIconName(); break;
        case 10: *reinterpret_cast< QXdgDBusImageVector*>(_v) = _t->overlayIconPixmap(); break;
        case 11: *reinterpret_cast< QString*>(_v) = _t->status(); break;
        case 12: *reinterpret_cast< QString*>(_v) = _t->title(); break;
        case 13: *reinterpret_cast< QXdgDBusToolTipStruct*>(_v) = _t->toolTip(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject thirdparty::QStatusNotifierItemAdaptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_meta_stringdata_thirdparty__QStatusNotifierItemAdaptor.data,
    qt_meta_data_thirdparty__QStatusNotifierItemAdaptor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *thirdparty::QStatusNotifierItemAdaptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *thirdparty::QStatusNotifierItemAdaptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_thirdparty__QStatusNotifierItemAdaptor.stringdata0))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int thirdparty::QStatusNotifierItemAdaptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 14;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 14;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 14;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 14;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 14;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void thirdparty::QStatusNotifierItemAdaptor::NewAttentionIcon()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void thirdparty::QStatusNotifierItemAdaptor::NewIcon()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void thirdparty::QStatusNotifierItemAdaptor::NewOverlayIcon()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void thirdparty::QStatusNotifierItemAdaptor::NewMenu()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void thirdparty::QStatusNotifierItemAdaptor::NewStatus(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void thirdparty::QStatusNotifierItemAdaptor::NewTitle()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void thirdparty::QStatusNotifierItemAdaptor::NewToolTip()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
